/**
 * @file
 * Javascript to handle diving trip view
 */

(function ($, _, Drupal, drupalSettings) {

  'use strict';
  
  var fieldsMetadataQueue = [];
  var fieldsAvailableQueue = [];
  
  // removes quickedit detach behavior to use our own
  Drupal.behaviors.quickedit.detach=null;
  
  function clickAgain(entityElement, entityModel, clickSourceElement) {
    if (entityModel.get('state') == 'opened') {
      $(clickSourceElement).click();
    }
    else {
      setTimeout(clickAgain, 20, entityElement, entityModel, clickSourceElement);
    }
  }
  
  function prepareInitializeField(fieldElement){
    var metadata = Drupal.quickedit.metadata;
    var fieldID = fieldElement.getAttribute('data-quickedit-field-id');
    var entityID = extractEntityID(fieldID);

    var entityElementSelector = '[data-quickedit-entity-id="' + entityID + '"]';
    var $entityElement = $(entityElementSelector);

    if (!$entityElement.length) {
      throw new Error('Quick Edit could not associate the rendered entity field markup (with [data-quickedit-field-id="' + fieldID + '"]) with the corresponding rendered entity markup: no parent DOM node found with [data-quickedit-entity-id="' + entityID + '"]. This is typically caused by the theme\'s template for this entity type forgetting to print the attributes.');
    }
    var entityElement = $(fieldElement).closest($entityElement);

    if (entityElement.length === 0) {
      var $lowestCommonParent = $entityElement.parents().has(fieldElement).first();
      entityElement = $lowestCommonParent.find($entityElement);
    }
    var entityInstanceID = entityElement.get(0).getAttribute('data-quickedit-entity-instance-id');

    if (!metadata.has(fieldID)) {
      setTimeout(prepareInitializeField, 200, fieldElement);
      return;
    }

    if (metadata.get(fieldID, 'access') !== true) {
      return;
    }

    // have the metadata  about the etnity been added yet?
    if (Drupal.quickedit.metadata.has(entityID)) {
      if (! Drupal.quickedit.collections.entities.findWhere({
        entityID: entityID,
        entityInstanceID: entityInstanceID
      })) {
        var entityModel = new Drupal.quickedit.EntityModel({
          el: entityElement.get(0),
          entityID: entityID,
          entityInstanceID: entityInstanceID,
          id: entityID + '[' + entityInstanceID + ']',
          label: Drupal.quickedit.metadata.get(entityID, 'label')
        });
        Drupal.quickedit.collections.entities.add(entityModel);

        var entityDecorationView = new Drupal.quickedit.EntityDecorationView({
          el: entityElement.get(0),
          model: entityModel
        });
        entityModel.set('entityDecorationView', entityDecorationView);
 
      }
      
      initializeField(fieldElement, fieldID, entityID, entityInstanceID);
    }
    else {
      setTimeout(prepareInitializeField, 200, fieldElement);
      return;
    }
    
  }
  
  function initializeField(fieldElement, fieldID, entityID, entityInstanceID) {
  
    // if the element has not yet been added
    if (! Drupal.quickedit.collections.fields.findWhere({
        id:  fieldID + '[' + entityInstanceID + ']'
     })) {
      $(fieldElement).addClass('quickedit-field');
      
      var entity = Drupal.quickedit.collections.entities.findWhere({
        entityID: entityID,
        entityInstanceID: entityInstanceID
      });
        
      var field = new Drupal.quickedit.FieldModel({
        el: fieldElement,
        fieldID: fieldID,
        id: fieldID + '[' + entityInstanceID + ']',
        entity: entity,
        metadata: Drupal.quickedit.metadata.get(fieldID),
        acceptStateChange: _.bind(Drupal.quickedit.app.acceptEditorStateChange, Drupal.quickedit.app)
      });

      Drupal.quickedit.collections.fields.add(field);
      
      // and add the editor
      loadMissingEditors();
    }
    
    // add the handlers
    if ($(fieldElement).hasClass('simpler_quickedit_click')) {
      addClickHandler(fieldElement);
    }
    if ($(fieldElement).is('.simpler_quickedit_edit, .simpler_quickedit_editHover')) {
      addEditHandler(fieldElement);
    }
  }
  
  function addClickHandler(fieldElement) {
    $(fieldElement).click(function(e) {
      let fieldID = $(this).attr('data-quickedit-field-id');
      let thisEntityID = extractEntityID(fieldID);
      let thisEntityElement = $(this).closest('[data-quickedit-entity-id="' + thisEntityID + '"]').get(0);
      var entityModel = Drupal.quickedit.collections.entities.findWhere({
        el: thisEntityElement
      });
      
      if ( ! entityModel) {
        console.log('issue with model');
      }
      
      if (entityModel && entityModel.get('state') == 'closed') {
        entityModel.set('state', 'launching');
        clickAgain(thisEntityElement, entityModel, this); 
      }
    });
  }
  
  function addEditHandler(fieldElement) {
    let edit = $('<div />').addClass('contextual simpler_quickedit-do');
    $('<button/>').addClass('trigger focusable').attr('type','button').html(Drupal.t('Edit')).appendTo(edit);
    
    // position absolutely on the middle of the parent
    $(fieldElement).css('position', 'relative');
    
    $(fieldElement).append(edit);
    edit.click(function(e) {
      let fieldID = $(this).parent().attr('data-quickedit-field-id');
      let thisEntityID = extractEntityID(fieldID);
      let thisEntityElement = $(this).closest('[data-quickedit-entity-id="' + thisEntityID + '"]').get(0);
      var entityModel = Drupal.quickedit.collections.entities.findWhere({
        el: thisEntityElement
      });
      
      if ( ! entityModel) {
        console.log('issue with model');
      }
      
      if (entityModel && entityModel.get('state') == 'closed') {
        entityModel.set('state', 'launching');
        clickAgain(thisEntityElement, entityModel, this); 
      }
    });
  }
  
  function loadMissingEditors() {
    var loadedEditors = _.keys(Drupal.quickedit.editors);
    var missingEditors = [];
    Drupal.quickedit.collections.fields.each(function (fieldModel) {
      var metadata = Drupal.quickedit.metadata.get(fieldModel.get('fieldID'));
      if (metadata.access && _.indexOf(loadedEditors, metadata.editor) === -1) {
        missingEditors.push(metadata.editor);

        Drupal.quickedit.editors[metadata.editor] = false;
      }
    });
    missingEditors = _.uniq(missingEditors);
    if (missingEditors.length === 0) {
      return;
    }

    var loadEditorsAjax = Drupal.ajax({
      url: Drupal.url('quickedit/attachments'),
      submit: { 'editors[]': missingEditors }
    });

    var realInsert = Drupal.AjaxCommands.prototype.insert;
    loadEditorsAjax.commands.insert = function (ajax, response, status) {
      realInsert(ajax, response, status);
    };

    loadEditorsAjax.execute();
  }

  
  function extractEntityID(fieldID) {
    return fieldID.split('/').slice(0, 2).join('/');
  }
  
  
  Drupal.behaviors.simplerquickedit = {
    attach: function (context, drupalSettings) {

      $('.simpler_quickedit', context).once().each(function() {
        prepareInitializeField(this);
      });
    },
    detach: function detach(context, settings, trigger) {
      /* couldn't find any satisfying way to detach elements - no detach for now */
    }
  }

  
}) (jQuery, _, Drupal, drupalSettings);